classdef SimSettings < handle
% class SimSettings
%
%   Maintains simulation settings. It keeps the settings parameters that 
%   are specific  to each simulation. It can have only one instance 
%   throughout the whole simulation thanks to Singleton Pattern. Its 
%   properties should be initialized once in the simulation and then used 
%   by other entities by  using the get() functions provided by it. 
%
%   See also initSimSettings.

%   Copyright  2017-2018 Mehmet Kurum, Orhan Eroglu, Dylan R. Boyd

%   This program is free software: You can redistribute it and/or 
%   modify it under the terms of the GNU General Public License as 
%   published by the Free Software Foundation, either version 3 of the 
%   License, or (at your option) any later version.

%   Version: 1.0.0


    
    properties (SetAccess = private, GetAccess = public)
        
        % Simulator version, e.g. "1.0"
        version = Constants.VERSION;
        
        % Campaign name, e.g. "MSU"
        campaign
        
        % Simulation name that is generated by using campaign info and the
        % execution time of the simulation
        sim_name
        
        % Simulator to be run
        % 1: SCoBi-Veg
        % 2: SCoBi-ML
        simulator_id
        
        % Simulation mode
        % 1: Snapshot
        % 2: Time-series
        sim_mode_id
        
        % Ground cover
        % 1: Bare-soil
        % 2: Vegetation
        gnd_cover_id
        
        % Flag for whether to write attenuation to Excel file or not
        % 0: Do not write 
        % 1: Write
        write_attenuation
        
        % Flag for including the simulation in Master Simulation file
        % 0: Do not include
        % 1: Include
        include_in_master_sim_file
        
    end
    
    
    methods (Access = private)
    
        function obj = SimSettings
            % SIMSETTINGS - Private constructor
        end
    
    end
    
    methods (Static)
        
        
        function singleObj = getInstance
            % GETINSTANCE - One instance for Singleton Pattern
             persistent localObj
             
             if isempty(localObj) || ~isvalid(localObj)
                localObj = SimSettings;
             end
             
             singleObj = localObj;
        end
        
    end
    
    
    methods
        
        function initialize(obj, campaign, simulator_id, sim_mode_id, gnd_cover_id, write_attenuation, ...
                include_in_master_sim_file )
            % INITIALIZE - Initializes all the properties
            
            obj.campaign = campaign;
            
            currentTime = datestr(datetime('now'));
            currentTime = strrep( currentTime, ' ', '-' );
            currentTime = strrep( currentTime, '-', '_' );
            currentTime = strrep( currentTime, ':', '_' );
            obj.sim_name = strcat( obj.campaign, '-', currentTime );
            
            obj.simulator_id = simulator_id;
            obj.sim_mode_id = sim_mode_id;
            obj.gnd_cover_id = gnd_cover_id;
            obj.write_attenuation = write_attenuation;
            obj.include_in_master_sim_file = include_in_master_sim_file;    
            
        end
        
        function out = get.campaign(obj)
            out = obj.campaign;
        end 
        
        function out = get.gnd_cover_id(obj)
            out = obj.gnd_cover_id;        
        end
        
        function out = get.include_in_master_sim_file(obj)
            out = obj.include_in_master_sim_file;
        end
        
        function out = get.sim_mode_id(obj)
            out = obj.sim_mode_id;        
        end
        
        function out = get.simulator_id(obj)
            out = obj.simulator_id;        
        end
        
        function out = get.version(obj)
            out = obj.version;        
        end
        
        function out = get.write_attenuation(obj)
            out = obj.write_attenuation;
        end 
    end
    
end

